local Excavator = Rock:NewAddon("Excavator", "LibRockTimer-1.0", "LibRockEvent-1.0", "LibRockConsole-1.0", "LibRockDB-1.0", "LibRockConfig-1.0")

local function InBG()
	for i = 1, MAX_BATTLEFIELD_QUEUES do
		local s = GetBattlefieldStatus(i)
		if s == "active" then
			return true
		end
	end
	return false
end

Excavator:SetDatabase("ExcavatorDB")

Excavator:SetDatabaseDefaults("profile", {
	delay = 300,
	ignorefriends = true,
	verbose = false,
} )

Excavator:SetConfigTable({
	type = "group",
	name = "Excavator",
	desc = "Excavator",
	args = {
		Delay = {
			order = 100,
			type = "number",
			name = "Report delay",
			desc = "Delay (in seconds) between each report",
			get = function() return Excavator.db.profile.delay end,
			set = function(value)
				Excavator.db.profile.delay = value
				if InBG() then
					Excavator:RemoveTimer("ExcavatorTimer")
					Excavator:EngageTimer()
				end
			end,
			min = 30,
			max = 300,
			step = 5,
			bigStep = 15,
		},
		IgnoreFriends = {
			order = 200,
			type = "boolean",
			name = "Skip Friends",
			desc = "Do not report friends or guildmates",
			get = function() return Excavator.db.profile.ignorefriends end,
			set = function() Excavator.db.profile.ignorefriends = not Excavator.db.profile.ignorefriends end,
		},
		Verbose = {
			order = 300,
			type = "boolean",
			name = "Verbose",
			desc = "Display number of reported players",
			get = function() return Excavator.db.profile.verbose end,
			set = function() Excavator.db.profile.verbose = not Excavator.db.profile.verbose end,
		},
	},
})

Excavator:SetConfigSlashCommand("/excavator", "/exc")
Excavator:AddSlashCommand("Report", "/excavate")

function Excavator:OnEnable()
	self:EngageTimer()
	self:AddEventListener("UPDATE_BATTLEFIELD_STATUS", "EngageTimer")
end

function Excavator:Report()

	if not InBG() then
		self:RemoveTimer("ExcavatorTimer")
		return
	end

	-- function for ChatFrame_AddMessageEventFilter
	-- hides "Notification sent for %s." and adds to reported count
	local reported = 0
	local friendsignored = 0
	local f = function(s)
		if string.gmatch(s, REPORT_PVP_AFK_SENT) then
			reported = reported + 1
			return true
		end
	end

	-- engage filter
	ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", f)

	if self.db.profile.ignorefriends then
		-- if ignore friends is enabled then we must build a table for friends/guildies
		local isfriend = { }

		-- add friends
		for i = 1, GetNumFriends() do
			local n, _, _, _, o = GetFriendInfo(i)
			if not o then -- offline friends are indexed after online friends, no sense wasting memory on offline players
				break
			else
				isfriend[n] = true
			end
		end

		-- add guildies
		local showoffline = GetGuildRosterShowOffline() -- to preserve "show offline" option
		SetGuildRosterShowOffline(false)

		for i = 1, GetNumGuildMembers(false) do
			local n = GetGuildRosterInfo(i)
			isfriend[n] = true
		end

		SetGuildRosterShowOffline(showoffline)

		-- and finally, iterate through raid members
		for i = 1, MAX_RAID_MEMBERS do
			local n, r = UnitName("raid"..i)
			if r ~= nil or not isfriend[n] then
				ReportPlayerIsPVPAFK("raid"..i)
			elseif UnitExists("raid"..i) then
				friendsignored = friendsignored + 1
			end
		end
	else
		for i = 1, MAX_RAID_MEMBERS do
			ReportPlayerIsPVPAFK("raid"..i)
		end
	end

	-- disengage filter
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", f)
	if self.db.profile.verbose then
		self:Print("Reported "..HIGHLIGHT_FONT_COLOR_CODE..reported..FONT_COLOR_CODE_CLOSE.." players. "..((friendsignored > 0) and ("Skipped "..HIGHLIGHT_FONT_COLOR_CODE..friendsignored..FONT_COLOR_CODE_CLOSE.." friends.") or ""))
	end

end

function Excavator:EngageTimer()
	if InBG() and not self:HasTimer("ExcavatorTimer") then
		self:AddRepeatingTimer("ExcavatorTimer", self.db.profile.delay, "Report")
	end
end